/**
 * File for struct logic.
 */


// User Import Headers.
#ifndef user_headers
    #define user_headers
    #include "structs.h"
#endif


// Global Variables.
const int tag_main_termination = 1;
const int tag_work_request = 2;
const int tag_status_update = 3;
thread_struct* thread_args_ptr;


/**
 * Writes a single line to log file for processor.
 */
void _log(char const* file, long line, char const* message) {

    // Print to console (debugging only).
    // printf(" P%3i [%s:%ld] %s\n", thread_args_ptr->thread_num, file, line, message);

    // Log to file.
    fprintf(thread_args_ptr->log_file, "[%s:%ld] %s\n", file, line, message);

    // char* msg_buffer = calloc(1024, sizeof(char));
    // sprintf(msg_buffer, "[%s:%ld] %s\n", file, line, message);
    // MPI_File* log_file = thread_args_ptr->log_file;
    // MPI_File_write(&log_file, msg_buffer, 1024, MPI_CHAR, MPI_STATUS_IGNORE);
    // free(msg_buffer);
}


/**
 * Initializes a "thread_struct" object.
 */
thread_struct* initialize_thread_struct(
        int total_processors,
        int seconds_per_load,
        int total_loads,
        int thread_num,
        int remaining_loads
    ) {

    // Create struct.
    thread_args_ptr = calloc(1, sizeof(thread_struct));

    // Populate fields.
    thread_args_ptr->total_processors = total_processors;
    thread_args_ptr->seconds_per_load = seconds_per_load;
    thread_args_ptr->total_loads = total_loads;
    thread_args_ptr->thread_num = thread_num;
    thread_args_ptr->remaining_loads = 0;

    // Create logging dir if doesn't exist.
    struct stat st = {0};
    if (stat("logging/", &st) == -1) {
        mkdir("logging/", 0700);
    }

    // Initialize logging location.
    char* log_location = calloc(256, sizeof(char));
    sprintf(log_location, "logging/thread_%i.log", thread_num);
    FILE* file_ptr = fopen(log_location, "w");
    // MPI_File* file_ptr = malloc(sizeof(MPI_File));
    // MPI_File_open(MPI_COMM_SELF, log_location, MPI_MODE_CREATE | MPI_MODE_WRONLY, MPI_INFO_NULL, file_ptr);

    // Check for errors.
    if (file_ptr == NULL) {
        printf("Error opening logging location at \"%s\".\n", log_location);
        free(log_location);
        free(thread_args_ptr);
        exit(1);
    } else {
        thread_args_ptr->log_file_location = log_location;
        thread_args_ptr->log_file = file_ptr;
    }

    log("");
    log("Initializing arg_struct.");
    return thread_args_ptr;
}


/**
 * Destroyes passed "thread_struct" object and all associated fields.
 */
void free_thread_struct(thread_struct* thread_args_ptr) {
    log("Destroying arg_struct.");

    fclose(thread_args_ptr->log_file);
    // MPI_File_close(thread_args_ptr->log_file);
    // free(thread_args_ptr->log_file);
    free(thread_args_ptr->log_file_location);
    free(thread_args_ptr);
}
